@@ -301,3 +301,16 @@ c     dim of rarray is implicitly declared.  SLIRP should warn of this.
       real rarray(nelems)
 
       end
+
+      subroutine sub_with_func_arg(funcptr,dummy)
+      external funcptr
+      integer dummy
+c     SLIRP will ignore this b/c function arguments are not supported yet
+      end
+
+      real function func_with_func_arg(funcptr,dummy)
+      external funcptr
+      integer dummy
+c     SLIRP will ignore this b/c function arguments are not supported yet
+      func_with_func_arg = 999.99
+      end
