@@ -1891,13 +1891,30 @@ gtk_plot_canvas_get_child_position(GtkPlotCanvas *canvas,
 gpointer
 gtk_plot_canvas_get_child_data(GtkPlotCanvasChild *child)
 {
-   return child->data;
+   if (child != NULL)
+	return child->data;
+
+   return NULL;
 }
 
+gboolean
+gtk_plot_canvas_child_toggle_slashed (GtkPlotCanvasChild *child)
+{
+   gboolean previous_setting = FALSE;
+   if (child != NULL) {
+	previous_setting = child->slashed;
+	child->slashed = !child->slashed;
+   }
+   return previous_setting;
+}
+
+
 GtkPlotCanvasType
 gtk_plot_canvas_get_child_type(GtkPlotCanvasChild *child)
 {
-   return child->type;
+   if (child != NULL)
+	return child->type;
+   return GTK_PLOT_CANVAS_NONE;
 }
 
 void
@@ -2327,7 +2344,7 @@ gtk_plot_canvas_put_rectangle(GtkPlotCanvas *canvas,
 			      const GdkColor *fg,
 			      const GdkColor *bg,
 			      GtkPlotBorderStyle border,
-			      gboolean fill)
+			      guint options)
 {
   GtkPlotCanvasChild *child;
   GtkPlotCanvasRectangle *rectangle;
@@ -2339,7 +2356,7 @@ gtk_plot_canvas_put_rectangle(GtkPlotCanvas *canvas,
   gdk_color_white(gdk_colormap_get_system(), &rectangle->bg); 
 
   gtk_plot_canvas_rectangle_set_attributes(child, style, width, 
-                                           fg, bg, border, fill);
+                                           fg, bg, border, options);
   rectangle->shadow_width = SHADOW_WIDTH;
 
   gtk_plot_canvas_put_child(canvas, child, x1, y1, x2, y2);
@@ -2354,7 +2371,7 @@ gtk_plot_canvas_put_ellipse(GtkPlotCanvas *canvas,
 			    gfloat width,
 			    const GdkColor *fg,
 			    const GdkColor *bg,
-			    gboolean fill)
+			    guint options)
 {
   GtkPlotCanvasChild *child;
   GtkPlotCanvasEllipse *ellipse;
@@ -2365,8 +2382,7 @@ gtk_plot_canvas_put_ellipse(GtkPlotCanvas *canvas,
   gdk_color_black(gdk_colormap_get_system(), &ellipse->line.color); 
   gdk_color_white(gdk_colormap_get_system(), &ellipse->bg); 
 
-  gtk_plot_canvas_ellipse_set_attributes(child, style, width, 
-                                         fg, bg, fill);
+  gtk_plot_canvas_ellipse_set_attributes(child, style, width, fg, bg, options);
 
   gtk_plot_canvas_put_child(canvas, child, x1, y1, x2, y2);
   return child;
@@ -2380,7 +2396,7 @@ gtk_plot_canvas_put_polygon(GtkPlotCanvas *canvas,
 			    gfloat width,
 			    const GdkColor *fg,
 			    const GdkColor *bg,
-			    gboolean fill)
+			    guint options)
 {
 
   GtkPlotCanvasChild *child;
@@ -2393,7 +2409,7 @@ gtk_plot_canvas_put_polygon(GtkPlotCanvas *canvas,
 
   gdk_color_black(gdk_colormap_get_system(), &poly->line.color); 
   gdk_color_white(gdk_colormap_get_system(), &poly->bg); 
-  gtk_plot_canvas_polygon_set_attributes(child, style, width, fg, bg, fill);
+  gtk_plot_canvas_polygon_set_attributes(child, style, width, fg, bg, options);
 
   /* Establish bounding box in canvas space */
   x1 = x2 = cpoints[0].x;
@@ -2419,7 +2435,7 @@ gtk_plot_canvas_put_polygon(GtkPlotCanvas *canvas,
 
   gdk_color_black(gdk_colormap_get_system(), &poly->line.color); 
   gdk_color_white(gdk_colormap_get_system(), &poly->bg); 
-  gtk_plot_canvas_polygon_set_attributes(child, style, width, fg, bg, fill);
+  gtk_plot_canvas_polygon_set_attributes(child, style, width, fg, bg, options);
 
   gtk_plot_canvas_put_child(canvas, child, x1, y1, x2, y2);
   return child;
@@ -2479,7 +2495,7 @@ gtk_plot_canvas_rectangle_set_attributes(GtkPlotCanvasChild *child,
 			                 const GdkColor *fg,
 			                 const GdkColor *bg,
 				         GtkPlotBorderStyle border,
-			                 gboolean fill)
+			                 guint options)
 {
   GtkPlotCanvasRectangle *rectangle;
 
@@ -2492,8 +2508,9 @@ gtk_plot_canvas_rectangle_set_attributes(GtkPlotCanvasChild *child,
     rectangle->bg = *bg;
   rectangle->line.line_width = width;
   rectangle->line.line_style = style;
-  rectangle->filled = fill;
   rectangle->border = border;
+  rectangle->filled = options & GTK_PLOT_CANVAS_CHILD_FILLED;
+  child->slashed = options & GTK_PLOT_CANVAS_CHILD_SLASHED;
 }
 
 void
@@ -2502,7 +2519,7 @@ gtk_plot_canvas_ellipse_set_attributes(GtkPlotCanvasChild *child,
 			               gfloat width,
 			               const GdkColor *fg,
 			               const GdkColor *bg,
-			               gboolean fill)
+			               guint options)
 {
   GtkPlotCanvasEllipse *ellipse;
 
@@ -2515,7 +2532,8 @@ gtk_plot_canvas_ellipse_set_attributes(GtkPlotCanvasChild *child,
     ellipse->bg = *bg;
   ellipse->line.line_width = width;
   ellipse->line.line_style = style;
-  ellipse->filled = fill;
+  ellipse->filled = options & GTK_PLOT_CANVAS_CHILD_FILLED;
+  child->slashed = options & GTK_PLOT_CANVAS_CHILD_SLASHED;
 }
 
 void
@@ -2524,7 +2542,7 @@ gtk_plot_canvas_polygon_set_attributes(GtkPlotCanvasChild *child,
 			               gfloat width,
 			               const GdkColor *fg,
 			               const GdkColor *bg,
-			               gboolean fill)
+			               guint options)
 {
   GtkPlotCanvasPolygon *poly;
 
@@ -2537,7 +2555,8 @@ gtk_plot_canvas_polygon_set_attributes(GtkPlotCanvasChild *child,
     poly->bg = *bg;
   poly->line.line_width = width;
   poly->line.line_style = style;
-  poly->filled = fill;
+  poly->filled = options & GTK_PLOT_CANVAS_CHILD_FILLED;
+  child->slashed = options & GTK_PLOT_CANVAS_CHILD_SLASHED;
 }
 
 GtkPlotCanvasChild *
@@ -2556,6 +2575,7 @@ gtk_plot_canvas_child_new(GtkPlotCanvasType type)
 
   child->selection = GTK_PLOT_CANVAS_SELECT_MARKERS;
   child->mode = GTK_PLOT_CANVAS_SELECT_CLICK_2;
+  child->slashed = FALSE;
 
   switch(type){
     case GTK_PLOT_CANVAS_LINE:
@@ -2709,6 +2729,22 @@ gtk_plot_canvas_child_set_selection_mode (GtkPlotCanvasChild *child,
 }
 
 /**********************************************************************/
+
+static void draw_slash(GtkPlotCanvas *canvas, gint x1, gint y1,gint x2, gint y2) /*{{{*/
+{
+   static GdkColor red;
+   static GdkColormap *colormap = NULL;
+
+   if (!colormap) {
+	gdk_color_parse("red", &red);
+	colormap = gdk_colormap_get_system();
+	gdk_color_alloc(colormap, &red);
+   }
+   gtk_plot_pc_set_lineattr(canvas->pc, 0, GDK_LINE_ON_OFF_DASH, 0, 0);
+   gtk_plot_pc_set_color(canvas->pc, &red);
+   gtk_plot_pc_draw_line(canvas->pc, x1, y1, x2, y2);
+} /*}}}*/
+
 static void
 gtk_plot_canvas_draw_child(GtkPlotCanvas *canvas,
                            GtkPlotCanvasChild *child)
@@ -2836,6 +2872,8 @@ gtk_plot_canvas_draw_child(GtkPlotCanvas *canvas,
                                  roundint(rectangle->shadow_width * m), height);
             }
         }
+	if (child->slashed)
+	   draw_slash(canvas, xmin, ymin, xmin+width, ymin+height);
         break;    
     case GTK_PLOT_CANVAS_ELLIPSE:
         ellipse = (GtkPlotCanvasEllipse *)child->data;
@@ -2848,6 +2886,17 @@ gtk_plot_canvas_draw_child(GtkPlotCanvas *canvas,
         if(ellipse->line.line_style != GTK_PLOT_LINE_NONE)
            gtk_plot_pc_draw_ellipse(canvas->pc, FALSE, 
                                    xmin, ymin, width, height);
+
+	if (child->slashed) {
+	   /* Draw exclude marker within ellipse using its parametric form */
+	   #define SQRT_2_OVER_2 0.70710678118654757274
+	   gdouble a = width / 2.0;	/* semimajor axis */
+	   gdouble b = height / 2.0;    /* semiminor axis */
+	   gdouble h = xmin + a;	/* center: (h, k); */
+	   gdouble k = ymin + b;
+	   draw_slash(canvas, h - a * SQRT_2_OVER_2, k - b * SQRT_2_OVER_2,
+				h + a * SQRT_2_OVER_2, k + b * SQRT_2_OVER_2);
+	}
         break;    
     case GTK_PLOT_CANVAS_POLYGON:
         poly = (GtkPlotCanvasPolygon*)child->data;
@@ -2872,6 +2921,8 @@ gtk_plot_canvas_draw_child(GtkPlotCanvas *canvas,
            gtk_plot_pc_draw_polygon(canvas->pc,FALSE,points,poly->npoints);
 
 	g_free(points);
+	if (child->slashed)
+	   draw_slash(canvas, xmin, ymin, xmin+width, ymin+height);
         break;    
     case GTK_PLOT_CANVAS_TEXT:
         text = (GtkPlotText *)child->data;
