@@ -4,7 +4,7 @@
  *
  * 	SLgtk:  S-Lang bindings for GTK+ widget set
  *
- * Copyright (C) 2003-2004 Massachusetts Institute of Technology 
+ * Copyright (C) 2003-2009 Massachusetts Institute of Technology 
  * Copyright (C) 2002 Michael S. Noble <mnoble@space.mit.edu>
  *
  */
@@ -44,6 +44,14 @@ SLang_CStruct_Field_Type GParamSpec_Layout [] =
   MAKE_CSTRUCT_FIELD(GParamSpec,param_id,"param_id",SLANG_UINT_TYPE,0),
   SLANG_END_CSTRUCT_TABLE
 };
+
+SLang_CStruct_Field_Type GTimeVal_Layout[] =
+{
+  MAKE_CSTRUCT_FIELD(GTimeVal, tv_sec, "tv_sec", SLANG_LONG_TYPE, 0),
+  MAKE_CSTRUCT_FIELD(GTimeVal, tv_usec, "tv_usec", SLANG_LONG_TYPE, 0),
+  SLANG_END_CSTRUCT_TABLE
+};
+
 /* }}} */
 
 /* GValue handling {{{ */
@@ -243,7 +251,7 @@ int pop_g_value(GValue *arg)
 }
 /*  }}} */
 
-/* Function callbacks {{{ */
+/* Signal callbacks {{{ */
 
 struct _slGClosure {
   GClosure closure;
@@ -254,8 +262,7 @@ struct _slGClosure {
 };
 typedef struct _slGClosure slGClosure;
 
-static void
-slg_closure_marshal(GClosure *closure,
+static void slg_closure_marshal(GClosure *closure, /*{{{*/
                 	GValue *return_value,
 			guint n_param_values,
 			const GValue *param_values,
@@ -310,10 +317,9 @@ slg_closure_marshal(GClosure *closure,
 		(size_t) MIN(strlen(pcl->function->name),192-strlen(msg)-1));
 	   error_terminate_main_loop(msg);
 	}
-}
+} /*}}}*/
 
-static void
-slg_closure_destroy(gpointer data, GClosure *cl)
+static void slg_closure_destroy(gpointer data, GClosure *cl) /*{{{*/
 {
    unsigned int i;
    slGClosure *pcl = (slGClosure*)cl;
@@ -329,11 +335,10 @@ slg_closure_destroy(gpointer data, GClosure *cl)
 	SLang_free_anytype(pcl->swap_data);
 
    SLang_free_function(pcl->function);
-}
+} /*}}}*/
 
-GClosure*
-slg_closure_new(SLang_Name_Type *function, SLang_Any_Type **args,
-      			unsigned int nargs, SLang_Any_Type *swap_data)
+GClosure* slg_closure_new(SLang_Name_Type *function,  /*{{{*/
+	SLang_Any_Type **args, unsigned int nargs, SLang_Any_Type *swap_data)
 {
    GClosure *closure;
 
@@ -346,10 +351,9 @@ slg_closure_new(SLang_Name_Type *function, SLang_Any_Type **args,
    ((slGClosure*)closure)->nargs = nargs;
 
   return closure;
-}
+} /*}}}*/
 
-int
-extract_slang_args(unsigned int nargs, SLang_Any_Type ***pargs)
+int extract_slang_args(unsigned int nargs, SLang_Any_Type ***pargs) /*{{{*/
 {
    SLang_Any_Type **args;
    SLang_Any_Type *arg;
@@ -374,17 +378,16 @@ extract_slang_args(unsigned int nargs, SLang_Any_Type ***pargs)
    }
    *pargs = args;
    return 0;
-}
+} /*}}}*/
 
-void free_slang_args(unsigned int nargs, SLang_Any_Type **args)
+void free_slang_args(unsigned int nargs, SLang_Any_Type **args) /*{{{*/
 {
    while (nargs > 0)
 	SLang_free_anytype(args[--nargs]);
    SLfree((char*)args);
-}
+} /*}}}*/
 
-static void
-sig_conn_generic(gboolean after, const char *usage)
+static void sig_conn_generic(gboolean after, char *usg) /*{{{*/
 {
    GObject *obj ;
    Slirp_Opaque *obj_o = NULL;
@@ -395,7 +398,7 @@ sig_conn_generic(gboolean after, const char *usage)
    SLang_Ref_Type *function_ref = NULL;
    unsigned int nargs;
 
-   if (usage_err(3, usage))
+   if (usage_err(3, usg))
 	return;
 
    nargs = SLang_Num_Function_Args - 3;
@@ -420,21 +423,20 @@ sig_conn_generic(gboolean after, const char *usage)
    SLang_free_opaque(obj_o);
    SLang_free_slstring(signame);
    SLang_push_integer(connid);
-}
+} /*}}}*/
 
-static void slg_signal_connect(void)
+static void slg_signal_connect(void) /*{{{*/
 {
    sig_conn_generic(0,"id = g_signal_connect(GObject,string,function_ref,...)");
-}
+} /*}}}*/
 
-static void slg_signal_connect_after(void)
+static void slg_signal_connect_after(void) /*{{{*/
 {
    sig_conn_generic(1,
-	 	"id = g_signal_connect_after(GObject,string,function_ref,...)");
-}
+   	"id = g_signal_connect_after(GObject,string,function_ref,...)");
+} /*}}}*/
 
-static void 
-slg_signal_connect_swapped(void)
+static void slg_signal_connect_swapped(void) /*{{{*/
 {
    int  connid = -1;
    GObject *obj;
@@ -447,7 +449,7 @@ slg_signal_connect_swapped(void)
    unsigned int nargs;
    
    if (usage_err(4,
-	"id = g_signal_connect_swapped(GObject,string,function_ref,data,...)"))
+   	"id = g_signal_connect_swapped(GObject,string,function_ref,data,...)"))
 	return;
 
    nargs = SLang_Num_Function_Args - 4;
@@ -474,7 +476,7 @@ slg_signal_connect_swapped(void)
    SLang_free_opaque(obj_o);
    SLang_free_slstring(signame);
    SLang_push_integer(connid);
-}
+} /*}}}*/
 
 			/* !!! currently ignores sig hdlr args */
 static void slg_signal_emit_by_name(Slirp_Opaque *obj, char *name)
@@ -483,7 +485,52 @@ static void slg_signal_emit_by_name(Slirp_Opaque *obj, char *name)
 }
 /* }}} */
 
-/* Wrappers {{{ */
+/* Timeout & idle callbacks {{{ */
+static unsigned int sl_g_timeout_add(void)
+{
+   unsigned int interval;
+   slGFunction *f = NULL;
+
+   if (	usage_err(2,"id = g_timeout_add(millis, func_ref [, arg1, ...])")
+				|| (f = function_pop(1)) == NULL
+				|| -1 == SLang_pop_uinteger(&interval)) {
+
+	function_destroy(f);
+	return 0;
+   }
+
+   return g_timeout_add_full( G_PRIORITY_DEFAULT, (guint32)interval,
+			(GSourceFunc)function_invoke, f, function_destroy);
+}
+
+static unsigned int sl_g_idle_add(void)
+{
+   slGFunction *f = NULL;
+
+   if ( usage_err(1,"id = g_idle_add(func_ref [, arg1, ...])")
+					|| (f = function_pop(0)) == NULL)
+	return 0;
+
+   return g_idle_add_full(G_PRIORITY_DEFAULT_IDLE,
+   			(GSourceFunc)function_invoke, f, function_destroy);
+}
+
+static unsigned int sl_g_idle_add_priority(void)
+{
+   slGFunction *f = NULL;
+   int prio;
+
+   if ( usage_err(2,"id = g_idle_add_priority(prio, func_ref [, arg1, ...])")
+			|| (f = function_pop(1)) == NULL
+			|| -1 == SLang_pop_integer(&prio)) 
+	return 0;
+
+   return g_idle_add_full(prio, (GSourceFunc)function_invoke,
+						f, function_destroy);
+}
+/* }}} */
+
+/* Manually-coded wrappers {{{ */
 static void slg_enum_get_value(Slirp_Opaque *o, gint *value)
 {
    GEnumValue *gev = g_enum_get_value(o->instance,*value);
@@ -646,6 +693,10 @@ static SLang_Intrin_Fun_Type Glib_GObject_Funcs[] =
   MAKE_INTRINSIC_0("g_param_spec_set_qdata", slg_param_spec_set_qdata,V),
   MAKE_INTRINSIC_0("g_param_spec_get_qdata", slg_param_spec_get_qdata,V),
   MAKE_INTRINSIC_0("g_param_spec_steal_qdata", slg_param_spec_steal_qdata,V),
+  MAKE_INTRINSIC_0("g_timeout_add", sl_g_timeout_add, UI),
+  MAKE_INTRINSIC_0("g_idle_add", sl_g_idle_add, UI),
+  MAKE_INTRINSIC_0("g_idle_add_priority", sl_g_idle_add_priority, UI),
+
   SLANG_END_INTRIN_FUN_TABLE
 };
 
