@@ -77,12 +77,12 @@ int		slgtk_debug;
 
 SLang_CStruct_Field_Type *GtkAllocation_Layout = GdkRectangle_Layout;
 
-typedef struct _slGtkFunction {
+typedef struct _slGFunction {
   SLang_Name_Type *function;
   void		  *widget;
   SLang_Any_Type  **args;
   unsigned int    nargs;
-} slGtkFunction;		/* Used for GtkFunction and GtkCallback */
+} slGFunction;		/* Used for signal, timer, idle, etc callbacks */
 /* }}} */
 
 /* Icon interface {{{ */
@@ -129,6 +129,8 @@ install_icons(void)
 }
 /* }}} */
 
+static void throw_gerror(GError *);
+
 /* GtkObject and GtkWidget manipulators {{{ */
 
 static void slgtk_object_destroyer(GtkObject *obj)
@@ -144,15 +146,6 @@ static void slgtk_object_destroyer(GtkObject *obj)
 #include "gtk_glue.c"
 #include "faccessors_glue.c"
 
-/* Gdk interface {{{ */
-#include "slgdk.c"
-#include "slgdkpixbuf.c"
-/*  }}} */
-
-/* Glib interface {{{ */
-#include "slglib.c"
-/*  }}} */
-
 static void sl_gtk_object_type(Slirp_Opaque *o)
 {
    unsigned long type = (unsigned long) GTK_OBJECT_TYPE(o->instance);
@@ -199,24 +192,141 @@ static void sl_gtk_widget_get_allocation(Slirp_Opaque *o)
 	return;
    SLang_verror(SL_USAGE_ERROR, "specified argument is not a GtkWidget*");
 }
-
 /* }}} */
 
-/* Unused terminal widget stuff {{{ */
-#ifdef ZVT
-static int fork_cmd(void)
+/* General signal/idle/timeout callback support {{{ */ 
+
+static int
+function_invoke(slGFunction *pf)
 {
-   return (int)fork();
+   unsigned int arg = 0;
+
+   if (SLang_start_arg_list() == -1)
+	return -1;
+
+   if (pf->widget && -1 == SLang_push_opaque(GtkWidget_Type,pf->widget,0))
+	return -1;
+
+   if (pf->args) {
+      for (arg = 0; arg < pf->nargs; arg++)
+	  if (SLang_push_anytype(pf->args[arg]) == -1)
+	     break;
+   }
+
+   if (SLang_get_error() || SLang_end_arg_list() == -1) {
+	SLdo_pop_n( (pf->widget != NULL) + arg);
+	return -1;
+   }
+
+   (void) SLexecute_function(pf->function);
+
+   /* Unrecoverable S-Lang errors should not cause main loop to hang */
+   if (SLang_get_error() < 0) {
+	error_terminate_main_loop(pf->function->name);
+	return -1;
+   }
+
+   /* Otherwise try our best to recover */
+   if (SLang_get_error()) {
+	SLang_restart(0);
+	SLang_set_error(0);
+   }
+
+   return 0;
 }
 
-static int execl_cmd(char *path, char *arg0, char *arg1, char* arg2)
+static void
+callback_invoker(GtkWidget *w, gpointer data)
 {
-   (void) arg1; (void) arg2;
-   return execl(path,arg0,NULL);
+   slGFunction *pf = (slGFunction*)data;
+   pf->widget = w;
+   (void) function_invoke(pf);
+}
+
+static void 
+function_marshaller (GtkObject *obj, gpointer data, guint n_args,GtkArg *args)
+{
+   int retval;
+   slGFunction *pf = (slGFunction*)data;
+
+   (void) obj; (void) n_args;
+
+   if (-1 == function_invoke(pf))
+	return;
+
+   if (SLang_pop_integer(&retval) == -1 || SLang_get_error() < 0) {
+	char msg[192];
+	strcpy(msg,"could not pop expected boolean return value from: ");
+	strncat(msg,pf->function->name,
+	      	(size_t) MIN(strlen(pf->function->name),192-strlen(msg)-1));
+	error_terminate_main_loop(msg);
+   }
+
+   *GTK_RETLOC_BOOL(args[0]) = retval ? TRUE : FALSE;
+}
+
+static void
+function_destroy (gpointer data)
+{
+   slGFunction *f = (slGFunction*)data;
+   if (f) {
+	free_slang_args(f->nargs, f->args);
+	SLang_free_function(f->function);
+	SLfree((char*)f);
+   }
+}
+
+static slGFunction*
+function_create(void *widget, SLang_Ref_Type **slfunc_ref,
+				SLang_Any_Type **args, unsigned int nargs)
+{
+   slGFunction *f;
+   SLang_Name_Type *slfunc;
+   
+   if ( (slfunc = SLang_get_fun_from_ref(*slfunc_ref)) == NULL)
+      return NULL;
+
+   SLang_free_ref(*slfunc_ref);
+   *slfunc_ref = NULL;
+
+   if ( (f = (slGFunction*) SLmalloc(sizeof(slGFunction))) == NULL) {
+	SLang_free_function(slfunc);
+	return NULL;
+   }
+
+   f->function	= slfunc;
+   f->widget	= (GtkWidget*)widget;
+   f->args	= args;
+   f->nargs	= nargs;
+
+   return f;
+}
+
+static slGFunction*
+function_pop(unsigned int num_args_to_omit)
+{
+   slGFunction *f;
+   SLang_Any_Type **args = NULL;
+   SLang_Ref_Type *func_ref = NULL;
+   unsigned int nargs = SLang_Num_Function_Args - num_args_to_omit - 1;
+
+   if (extract_slang_args(nargs,&args) == 0
+      		 && SLang_pop_ref(&func_ref) == 0
+		 && (f = function_create(NULL, &func_ref, args, nargs)))
+	return f;
+
+   if (args) free_slang_args(nargs,args);
+   if (func_ref) SLang_free_ref(func_ref);
+
+   return NULL;
 }
-#endif
 /* }}} */
 
+/* Gdk interface {{{ */
+#include "slgdk.c"
+#include "slgdkpixbuf.c"
+/*  }}} */
+
 /* GtkTextIter interface {{{ */
 
 static void sl_gtk_text_buffer_get_iter_at_line_offset ( /*{{{*/
@@ -671,137 +781,11 @@ static void sl_gtk_toolbar_append_item(void)
 }
 /* }}} */
 
-/* Callback Support {{{ */ 
-
-static int
-function_invoke(slGtkFunction *pf)
-{
-   unsigned int arg = 0;
-
-   if (SLang_start_arg_list() == -1)
-	return -1;
-
-   if (pf->widget && -1 == SLang_push_opaque(GtkWidget_Type,pf->widget,0))
-	return -1;
-
-   if (pf->args) {
-      for (arg = 0; arg < pf->nargs; arg++)
-	  if (SLang_push_anytype(pf->args[arg]) == -1)
-	     break;
-   }
-
-   if (SLang_get_error() || SLang_end_arg_list() == -1) {
-	SLdo_pop_n( (pf->widget != NULL) + arg);
-	return -1;
-   }
-
-   (void) SLexecute_function(pf->function);
-
-   /* Unrecoverable S-Lang errors should not cause main loop to hang */
-   if (SLang_get_error() < 0) {
-	error_terminate_main_loop(pf->function->name);
-	return -1;
-   }
-
-   /* Otherwise try our best to recover */
-   if (SLang_get_error()) {
-	SLang_restart(0);
-	SLang_set_error(0);
-   }
-
-   return 0;
-}
-
-static void
-callback_invoker(GtkWidget *w, gpointer data)
-{
-   slGtkFunction *pf = (slGtkFunction*)data;
-   pf->widget = w;
-   (void) function_invoke(pf);
-}
-
-static void 
-function_marshaller (GtkObject *obj, gpointer data, guint n_args,GtkArg *args)
-{
-   int retval;
-   slGtkFunction *pf = (slGtkFunction*)data;
-
-   (void) obj; (void) n_args;
-
-   if (-1 == function_invoke(pf))
-	return;
-
-   if (SLang_pop_integer(&retval) == -1 || SLang_get_error() < 0) {
-	char msg[192];
-	strcpy(msg,"could not pop expected boolean return value from: ");
-	strncat(msg,pf->function->name,
-	      	(size_t) MIN(strlen(pf->function->name),192-strlen(msg)-1));
-	error_terminate_main_loop(msg);
-   }
-
-   *GTK_RETLOC_BOOL(args[0]) = retval ? TRUE : FALSE;
-}
-
-static void
-function_destroy (gpointer data)
-{
-   slGtkFunction *f = (slGtkFunction*)data;
-   if (f) {
-	free_slang_args(f->nargs, f->args);
-	SLang_free_function(f->function);
-	SLfree((char*)f);
-   }
-}
-
-static slGtkFunction*
-function_create(void *widget, SLang_Ref_Type **slfunc_ref,
-				SLang_Any_Type **args, unsigned int nargs)
-{
-   slGtkFunction *f;
-   SLang_Name_Type *slfunc;
-   
-   if ( (slfunc = SLang_get_fun_from_ref(*slfunc_ref)) == NULL)
-      return NULL;
-
-   SLang_free_ref(*slfunc_ref);
-   *slfunc_ref = NULL;
-
-   if ( (f = (slGtkFunction*) SLmalloc(sizeof(slGtkFunction))) == NULL) {
-	SLang_free_function(slfunc);
-	return NULL;
-   }
-
-   f->function	= slfunc;
-   f->widget	= (GtkWidget*)widget;
-   f->args	= args;
-   f->nargs	= nargs;
-
-   return f;
-}
-
-static slGtkFunction*
-function_pop(unsigned int num_args_to_omit)
-{
-   slGtkFunction *f;
-   SLang_Any_Type **args = NULL;
-   SLang_Ref_Type *func_ref = NULL;
-   unsigned int nargs = SLang_Num_Function_Args - num_args_to_omit - 1;
-
-   if (extract_slang_args(nargs,&args) == 0
-      		 && SLang_pop_ref(&func_ref) == 0
-		 && (f = function_create(NULL, &func_ref, args, nargs)))
-	return f;
-
-   if (args) free_slang_args(nargs,args);
-   if (func_ref) SLang_free_ref(func_ref);
-
-   return NULL;
-}
-
+/* Gtk timeout, idle, etc callbacks {{{ */
 static unsigned int sl_gtk_timeout_add(void)
 {
    unsigned int interval;
-   slGtkFunction *f = NULL;
+   slGFunction *f = NULL;
 
    if (	usage_err(2,"id = gtk_timeout_add(millis, func_ref [, arg1, ...])")
 				|| (f = function_pop(1)) == NULL
@@ -817,7 +801,7 @@ static unsigned int sl_gtk_timeout_add(void)
 
 static unsigned int sl_gtk_idle_add(void)
 {
-   slGtkFunction *f = NULL;
+   slGFunction *f = NULL;
 
    if ( usage_err(1,"id = gtk_idle_add(func_ref [, arg1, ...])")
 					|| (f = function_pop(0)) == NULL)
@@ -829,7 +813,7 @@ static unsigned int sl_gtk_idle_add(void)
 
 static unsigned int sl_gtk_idle_add_priority(void)
 {
-   slGtkFunction *f = NULL;
+   slGFunction *f = NULL;
    int prio;
 
    if ( usage_err(2,"id = gtk_idle_add_priority(prio, func_ref [, arg1, ...])")
@@ -843,7 +827,7 @@ static unsigned int sl_gtk_idle_add_priority(void)
 
 static unsigned int sl_gtk_quit_add(void)
 {
-   slGtkFunction *f = NULL;
+   slGFunction *f = NULL;
    guint main_level;
 
    if (	usage_err(2,"id = gtk_quit_add(main_level, func_ref [,arg1, ...])")
@@ -861,7 +845,7 @@ static unsigned int sl_gtk_quit_add(void)
 static void sl_gtk_container_foreach(void)
 {
    Slirp_Opaque *c_o = NULL;
-   slGtkFunction *f = NULL;
+   slGFunction *f = NULL;
    GtkContainer *c;
 
    if (	usage_err(1,
@@ -903,6 +887,10 @@ static void sl_gtk_main_quit_when_idle (void)
 
 /* }}} */
 
+/* Glib interface {{{ */
+#include "slglib.c"
+/*  }}} */
+
 /* Miscellaneous {{{ */
 
 static void swallow_error (char* msg) { (void)msg; }
@@ -1060,11 +1048,6 @@ static SLang_Intrin_Fun_Type GtkWidget_Funcs[] =
 
 #include "faccessors_ftable.c"
 
-#ifdef ZVT
-  MAKE_INTRINSIC_0("fork",fork_cmd,I),
-  MAKE_INTRINSIC_4("execl",execl_cmd,I,S,S,S,S),
-#endif
-
   MAKE_INTRINSIC_0("_toggle_error_hook",toggle_error_hook,V),
 
   SLANG_END_INTRIN_FUN_TABLE
@@ -1129,7 +1112,7 @@ static SLang_IConstant_Type Generic_IConsts [] =
   SLANG_END_ICONST_TABLE
 };
 
-#ifdef __CYGWIN__
+#if defined(__CYGWIN__)
 static unsigned int _gtk_major_version;
 static unsigned int _gtk_minor_version;
 static unsigned int _gtk_micro_version;
@@ -1144,8 +1127,7 @@ static SLang_Intrin_Var_Type Global_Intrin_Vars[] =
 
 static SLang_Intrin_Var_Type Local_Intrin_Vars[] =
 {
-
-#if __CYGWIN__
+#if defined(__CYGWIN__)
   MAKE_VARIABLE("_gtk_major_version", &_gtk_major_version,UI,1),
   MAKE_VARIABLE("_gtk_minor_version", &_gtk_minor_version,UI,1),
   MAKE_VARIABLE("_gtk_micro_version", &_gtk_micro_version,UI,1),
@@ -1238,7 +1220,7 @@ int init_gtk_module_ns(char *ns_name)
 
    if (ns_name != NULL) {
 	ns = SLns_create_namespace (ns_name);
-	if (ns == NULL)
+	if (ns == NULL || (slns = SLmalloc(strlen(ns_name)+1)) == NULL)
 	   return -1;
    }
 
@@ -1275,6 +1257,13 @@ int init_gtk_module_ns(char *ns_name)
    if (init_sl_gtk(ns) == -1)
       return -1;
 
+   if ( gtk_major_version != GTK_MAJOR_VERSION ||
+	gtk_minor_version != GTK_MINOR_VERSION ||
+	gtk_micro_version != GTK_MICRO_VERSION)
+	fprintf(stderr, "Warning: SLgtk compiled with Gtk %d.%d.%d but runtime linked to version %d.%d.%d\n",
+		GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION,
+		gtk_major_version, gtk_minor_version, gtk_micro_version);
+
    return 0;
 }
 /* }}} */
