@@ -238,7 +238,7 @@ int init_gtkextra_module_ns(char *ns_name)
    if (slang_abi_mismatch()) return -1;
 
    ns = SLns_create_namespace (ns_name);
-   if (ns == NULL)
+   if (ns == NULL  || (slns = SLmalloc(strlen(ns_name)+1)) == NULL)
 	return -1;
 
    slirp_debug_pause("gtkextra");
